<?php
/**
 * Created by PhpStorm for zanupf-new-
 * User: vincenthguyo
 * Date: 07/10/2024
 * Time: 13:23
 */

namespace App\Http\Controllers\API;

use App\Models\Member;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Symfony\Component\HttpFoundation\Response;

class MemberController extends Controller
{
    public function index()
    {
        $members = Member::all();
        $response = [
            'status' => true,
            'message' => '',
            'data' => [
                'members' => $members,
            ]
        ];
        return response($response, Response::HTTP_OK);
    }

    public function store(Request $request)
    {
        // Validation
        $validated = $request->validate([
            'first_name' => 'required|string',
            'last_name' => 'required|string',
            'gender' => 'required|string|in:male,female,other',
            'date_of_birth' => 'required|date',
            'identification_number' => 'required|string',
            'identification_type_id' => 'required|integer',
            'phone_number' => 'required|string',
            'email' => 'nullable|email',
            'address' => 'nullable|string',
            'province_id' => 'nullable|integer|exists:provinces,id',
            'admin_district_id' => 'nullable|integer|exists:admin_districts,id',
            'party_district_id' => 'nullable|integer|exists:party_districts,id',
            'branch_id' => 'nullable|integer|exists:branches,id',
            'cell_id' => 'nullable|integer|exists:cells,id',
            'polling_station_id' => 'nullable|integer|exists:polling_stations,id',
            'wing_id' => 'required|integer|exists:wings,id',
            'card_fee' => 'required|numeric',
            'initial_subscription_fee' => 'required|numeric',
            'monthly_subscription_fee' => 'required|numeric',
        ]);
        $validated['pin'] = Hash::make(rand(1000, 9999));

        // Create the member
        $member = Member::create($validated);

        $response = [
            'status' => true,
            'message' => 'Member created successfully.',
            'data' => [
                'member' => $member,
            ]
        ];
        return response($response, Response::HTTP_CREATED);
    }

    public function show($id)
    {
        $member = Member::where('id', '=', $id)->first();
        if (is_null($member)) {
            $response = [
                'status' => false,
                'message' => 'Member not found.',
                'data' => []
            ];
            return response($response, Response::HTTP_NOT_FOUND);
        }

        $response = [
            'status' => true,
            'message' => '',
            'data' => [
                'member' => $member,
            ]
        ];
        return response($response, Response::HTTP_OK);
    }

    public function update(Request $request, $id)
    {
        $member = Member::where('id', '=', $id)->first();
        if (is_null($member)) {
            $response = [
                'status' => false,
                'message' => 'Member not found.',
                'data' => []
            ];
            return response($response, Response::HTTP_NOT_FOUND);
        }

        $validated = $request->validate([
            'first_name' => 'string',
            'last_name' => 'string',
            'gender' => 'string|in:male,female,other',
            'date_of_birth' => 'date',
            'identification_number' => 'string',
            'identification_type_id' => 'integer',
            'phone_number' => 'string',
            'email' => 'nullable|email',
            'address' => 'nullable|string',
            'province_id' => 'nullable|integer|exists:provinces,id',
            'admin_district_id' => 'nullable|integer|exists:admin_districts,id',
            'party_district_id' => 'nullable|integer|exists:party_districts,id',
            'branch_id' => 'nullable|integer|exists:branches,id',
            'cell_id' => 'nullable|integer|exists:cells,id',
            'polling_station_id' => 'nullable|integer|exists:polling_stations,id',
            'wing_id' => 'integer|exists:wings,id',
            'card_fee' => 'numeric',
            'initial_subscription_fee' => 'numeric',
            'monthly_subscription_fee' => 'numeric',
        ]);

        // Update member
        $member->update($validated);
        $member->refresh();

        $response = [
            'status' => true,
            'message' => 'Member updated successfully.',
            'data' => [
                'member' => $member,
            ]
        ];
        return response($response, Response::HTTP_OK);
    }

    public function destroy($id)
    {
        $member = Member::where('id', '=', $id)->first();
        if (is_null($member)) {
            $response = [
                'status' => false,
                'message' => 'Member not found.',
                'data' => []
            ];
            return response($response, Response::HTTP_NOT_FOUND);
        }

        $member->delete();
        $response = [
            'status' => true,
            'message' => 'Member deleted successfully.',
            'data' => []
        ];
        return response($response, Response::HTTP_OK);
    }
}
