<?php
/**
 * Created by PhpStorm for zanupf-new-
 * User: vincenthguyo
 * Date: 07/10/2024
 * Time: 13:02
 */

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Notifications\Notifiable;
use Laravel\Passport\HasApiTokens;
use Illuminate\Foundation\Auth\User as Authenticatable;

class Member extends Authenticatable
{
    use HasApiTokens, Notifiable;

    protected $fillable = [
        'first_name', 'last_name', 'gender', 'date_of_birth', 'identification_number', 'pin',
        'identification_type_id', 'phone_number', 'email', 'address', 'province_id',
        'admin_district_id', 'party_district_id', 'branch_id', 'cell_id', 'polling_station_id',
        'is_registered_to_vote', 'wing_id', 'card_fee', 'initial_subscription_fee', 'monthly_subscription_fee'
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'pin',
    ];

    // Relationships
    public function province()
    {
        return $this->belongsTo(Province::class);
    }

    public function adminDistrict()
    {
        return $this->belongsTo(AdminDistrict::class);
    }

    public function partyDistrict()
    {
        return $this->belongsTo(PartyDistrict::class);
    }

    public function branch()
    {
        return $this->belongsTo(Branch::class);
    }

    public function cell()
    {
        return $this->belongsTo(Cell::class);
    }

    public function pollingStation()
    {
        return $this->belongsTo(PollingStation::class);
    }

    public function wing()
    {
        return $this->belongsTo(Wing::class);
    }

    public function positions()
    {
        return $this->belongsToMany(Position::class)->withTimestamps();
    }

    public function payments()
    {
        return $this->hasMany(Payment::class);
    }

    public function transfers()
    {
        return $this->hasMany(Transfer::class);
    }

    // Methods
    public function isVotingEligible()
    {
        return $this->is_registered_to_vote;
    }

    public function currentPosition()
    {
        return $this->positions()->orderBy('created_at', 'desc')->first();
    }

    public function totalArrears()
    {
        return $this->payments()->where('status', 'unpaid')->sum('amount');
    }
}
