<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::create('members', function (Blueprint $table) {
            $table->id();
            $table->string('first_name');
            $table->string('last_name');
            $table->enum('gender', ['male', 'female', 'other']);
            $table->date('date_of_birth');
            $table->string('pin');
            $table->string('identification_number');
            $table->unsignedBigInteger('identification_type_id');
            $table->string('phone_number');
            $table->string('email')->nullable();
            $table->string('address')->nullable();
            $table->boolean('is_registered_to_vote')->default(false);
            $table->unsignedBigInteger('province_id')->nullable();
            $table->unsignedBigInteger('admin_district_id')->nullable();
            $table->unsignedBigInteger('party_district_id')->nullable();
            $table->unsignedBigInteger('branch_id')->nullable();
            $table->unsignedBigInteger('cell_id')->nullable();
            $table->unsignedBigInteger('polling_station_id')->nullable();
            $table->unsignedBigInteger('wing_id')->nullable();
            $table->decimal('card_fee', 8, 2)->default(0);
            $table->decimal('initial_subscription_fee', 8, 2)->default(0);
            $table->decimal('monthly_subscription_fee', 8, 2)->default(0);
            $table->timestamps();

            // Foreign key constraints
            $table->foreign('province_id')->references('id')->on('provinces')->onDelete('set null');
            $table->foreign('admin_district_id')->references('id')->on('admin_districts')->onDelete('set null');
            $table->foreign('party_district_id')->references('id')->on('party_districts')->onDelete('set null');
            $table->foreign('branch_id')->references('id')->on('branches')->onDelete('set null');
            $table->foreign('cell_id')->references('id')->on('cells')->onDelete('set null');
            $table->foreign('polling_station_id')->references('id')->on('polling_stations')->onDelete('set null');
            $table->foreign('wing_id')->references('id')->on('wings')->onDelete('set null');
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::dropIfExists('members');
    }
};
