<?php

use App\Http\Controllers\API\ApiAuthController;
use App\Http\Controllers\API\IndetificationTypeController;
use App\Http\Middleware\JsonResponse;
use Illuminate\Support\Facades\Route;

use App\Http\Controllers\API\MemberController;
use App\Http\Controllers\API\WingController;
use App\Http\Controllers\API\ProvinceController;
use App\Http\Controllers\API\AdminDistrictController;
use App\Http\Controllers\API\PartyDistrictController;
use App\Http\Controllers\API\BranchController;
use App\Http\Controllers\API\CellController;
use App\Http\Controllers\API\PollingStationController;
use App\Http\Controllers\API\PaymentController;
use App\Http\Controllers\API\TransferController;

Route::group(['middleware' => [JsonResponse::class]], function () {

    Route::prefix('v1')->group(function () {
        Route::post('/login', [ApiAuthController::class, 'login'])->name('login.api');
        Route::post('/register', [ApiAuthController::class, 'register'])->name('register.api');

        // Retrieve information for Provinces, Districts, Branches, Cells, etc.
        Route::get('provinces', [ProvinceController::class, 'index']);
        Route::get('admin-districts/{province}', [AdminDistrictController::class, 'index']);
        Route::get('party-districts/{adminDistrict}', [PartyDistrictController::class, 'index']);
        Route::get('branches/{partyDistrict}', [BranchController::class, 'index']);
        Route::get('cells/{branch}', [CellController::class, 'index']);
        Route::get('polling-stations/{cell}', [PollingStationController::class, 'index']);

        // Wings
        Route::get('wings', [WingController::class, 'index']);
        Route::get('identification-types', [IndetificationTypeController::class, 'index']);

        Route::middleware('auth:api')->group(function () {

            Route::get('/my-account', [ApiAuthController::class, 'getMyProfile'])->name('my.profile')->middleware('throttle:3,10');
            Route::post('/update-my-pin', [ApiAuthController::class, 'updateMyPassword'])->name('update.my.password');
            Route::put('/update-my-profile', [ApiAuthController::class, 'updateMyProfile'])->name('update.user.profile');
            Route::post('/delete-account', [ApiAuthController::class, 'requestAccountDeletion'])->name('delete.account');

            // Member registration and management
            Route::post('update-member/{member}', [MemberController::class, 'update']);
            Route::apiResource('members', MemberController::class);

            // Payments
            Route::get('members/{member}/payments', [PaymentController::class, 'index']);
            Route::post('members/{member}/payments', [PaymentController::class, 'store']);

            // Transfers
            Route::get('members/{member}/transfers', [TransferController::class, 'index']);
            Route::post('members/{member}/transfers', [TransferController::class, 'store']);
        });
    });

});
